/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "HTMLElement.h"
#include "HTMLTableElement.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomHTMLElement.h"
#include "gdom/GdomHTMLTableElement.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomHTMLElement.h"
#include "gdom/PrivateGdomHTMLTableElement.h"
#include "gdom/PrivateIntGdomHTMLElement.h"
#include "gdom/PrivateIntGdomHTMLTableElement.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMHTMLTableElement::GDOMHTMLTableElement(PassRefPtr<WebCore::HTMLTableElement> impl)
    : GDOMHTMLElement(impl)
{
}

GdomHTMLElement *
gdom_html_table_element_create_t_head (GdomHTMLTableElement *thiz)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::HTMLTableElement * item = (gdom_html_table_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    PassRefPtr<WebCore::HTMLElement> g_res = WTF::getPtr(item->createTHead());
    GdomHTMLElement * res = static_cast<GdomHTMLElement *>(WebCore::toGDOM(g_res.get()));
    return res;

}

void
gdom_html_table_element_delete_t_head (GdomHTMLTableElement *thiz)
{
    g_return_if_fail (thiz);
    WebCore::HTMLTableElement * item = (gdom_html_table_element_get_private(thiz));
    g_return_if_fail (item);
    item->deleteTHead();

}

GdomHTMLElement *
gdom_html_table_element_create_t_foot (GdomHTMLTableElement *thiz)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::HTMLTableElement * item = (gdom_html_table_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    PassRefPtr<WebCore::HTMLElement> g_res = WTF::getPtr(item->createTFoot());
    GdomHTMLElement * res = static_cast<GdomHTMLElement *>(WebCore::toGDOM(g_res.get()));
    return res;

}

void
gdom_html_table_element_delete_t_foot (GdomHTMLTableElement *thiz)
{
    g_return_if_fail (thiz);
    WebCore::HTMLTableElement * item = (gdom_html_table_element_get_private(thiz));
    g_return_if_fail (item);
    item->deleteTFoot();

}

GdomHTMLElement *
gdom_html_table_element_create_caption (GdomHTMLTableElement *thiz)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::HTMLTableElement * item = (gdom_html_table_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    PassRefPtr<WebCore::HTMLElement> g_res = WTF::getPtr(item->createCaption());
    GdomHTMLElement * res = static_cast<GdomHTMLElement *>(WebCore::toGDOM(g_res.get()));
    return res;

}

void
gdom_html_table_element_delete_caption (GdomHTMLTableElement *thiz)
{
    g_return_if_fail (thiz);
    WebCore::HTMLTableElement * item = (gdom_html_table_element_get_private(thiz));
    g_return_if_fail (item);
    item->deleteCaption();

}

GdomHTMLElement *
gdom_html_table_element_insert_row (GdomHTMLTableElement *thiz, glong index)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::HTMLTableElement * item = (gdom_html_table_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::HTMLElement> g_res = WTF::getPtr(item->insertRow(index, ec));
    GdomHTMLElement * res = static_cast<GdomHTMLElement *>(WebCore::toGDOM(g_res.get()));
    return res;

}

void
gdom_html_table_element_delete_row (GdomHTMLTableElement *thiz, glong index)
{
    g_return_if_fail (thiz);
    WebCore::HTMLTableElement * item = (gdom_html_table_element_get_private(thiz));
    g_return_if_fail (item);
    WebCore::ExceptionCode ec = 0;
    item->deleteRow(index, ec);

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "HTMLTableElement.h"
#include <gdom/GdomHTMLTableElement.h>
#include <gdom/PrivateGdomHTMLTableElement.h>
#include <gdom/PrivateIntGdomHTMLTableElement.h>
#include <gdom/GdomHTMLTableElementprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomHTMLTableElement, gdom_html_table_element, GDOM_TYPE_HTML_ELEMENT);

/*
struct _GdomHTMLTableElementPrivate {
    class WebCore::HTMLTableElement *item;
};

#define GDOM_HTML_TABLE_ELEMENT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_HTML_TABLE_ELEMENT, GdomHTMLTableElementPrivate))
*/
G_END_DECLS
WebCore::HTMLTableElement *
gdom_html_table_element_get_private(GdomHTMLTableElement *request)
{
    GDOMHTMLTableElement *res;
    res = static_cast<GDOMHTMLTableElement*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::HTMLTableElement*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_CAPTION,
    PROP_T_HEAD,
    PROP_T_FOOT,
    PROP_ROWS,
    PROP_T_BODIES,
    PROP_ALIGN,
    PROP_BG_COLOR,
    PROP_BORDER,
    PROP_CELL_PADDING,
    PROP_CELL_SPACING,
    PROP_FRAME,
    PROP_RULES,
    PROP_SUMMARY,
    PROP_WIDTH,
};


static void gdom_html_table_element_finalize(GObject* object)
{
    /*GdomHTMLTableElement* request = GDOM_HTML_TABLE_ELEMENT(object);*/
    /*GdomHTMLTableElementPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_HTML_ELEMENT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_html_table_element_parent_class)->finalize(object);
}

static void gdom_html_table_element__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomHTMLTableElement *self = GDOM_HTML_TABLE_ELEMENT(object);
    WebCore::HTMLTableElement * gself = (gdom_html_table_element_get_private(self));
    switch (prop_id) {
     case PROP_ALIGN:
    {
         gself->setAlign(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_BG_COLOR:
    {
         gself->setBgColor(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_BORDER:
    {
         gself->setBorder(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_CELL_PADDING:
    {
         gself->setCellPadding(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_CELL_SPACING:
    {
         gself->setCellSpacing(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_FRAME:
    {
         gself->setFrame(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_RULES:
    {
         gself->setRules(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_SUMMARY:
    {
         gself->setSummary(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_WIDTH:
    {
         gself->setWidth(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_html_table_element__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomHTMLTableElement *self = GDOM_HTML_TABLE_ELEMENT(object);
    WebCore::HTMLTableElement * gself = (gdom_html_table_element_get_private(self));
    switch (prop_id) {
    case PROP_CAPTION:
    {
         PassRefPtr<WebCore::HTMLTableCaptionElement> ptr = gself->caption();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_T_HEAD:
    {
         PassRefPtr<WebCore::HTMLTableSectionElement> ptr = gself->tHead();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_T_FOOT:
    {
         PassRefPtr<WebCore::HTMLTableSectionElement> ptr = gself->tFoot();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_ROWS:
    {
         PassRefPtr<WebCore::HTMLCollection> ptr = gself->rows();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_T_BODIES:
    {
         PassRefPtr<WebCore::HTMLCollection> ptr = gself->tBodies();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_ALIGN:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->align()));
         break;
    }
    case PROP_BG_COLOR:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->bgColor()));
         break;
    }
    case PROP_BORDER:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->border()));
         break;
    }
    case PROP_CELL_PADDING:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->cellPadding()));
         break;
    }
    case PROP_CELL_SPACING:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->cellSpacing()));
         break;
    }
    case PROP_FRAME:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->frame()));
         break;
    }
    case PROP_RULES:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->rules()));
         break;
    }
    case PROP_SUMMARY:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->summary()));
         break;
    }
    case PROP_WIDTH:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->width()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_html_table_element_class_init(GdomHTMLTableElementClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_html_table_element_finalize;
    g_klass->set_property = gdom_html_table_element__set_property_fn;
    g_klass->get_property = gdom_html_table_element__get_property_fn;

    g_object_class_install_property (g_klass, PROP_CAPTION,
            g_param_spec_object ("caption", /* name */
                "html_table_element_caption", /* short description */
                "read-write  GdomHTMLTableCaptionElement * HTMLTableElement.caption", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_HTML_TABLE_CAPTION_ELEMENT, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_T_HEAD,
            g_param_spec_object ("t_head", /* name */
                "html_table_element_t_head", /* short description */
                "read-write  GdomHTMLTableSectionElement * HTMLTableElement.t_head", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_HTML_TABLE_SECTION_ELEMENT, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_T_FOOT,
            g_param_spec_object ("t_foot", /* name */
                "html_table_element_t_foot", /* short description */
                "read-write  GdomHTMLTableSectionElement * HTMLTableElement.t_foot", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_HTML_TABLE_SECTION_ELEMENT, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_ROWS,
            g_param_spec_object ("rows", /* name */
                "html_table_element_rows", /* short description */
                "read-only  GdomHTMLCollection * HTMLTableElement.rows", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_HTML_COLLECTION, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_T_BODIES,
            g_param_spec_object ("t_bodies", /* name */
                "html_table_element_t_bodies", /* short description */
                "read-only  GdomHTMLCollection * HTMLTableElement.t_bodies", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_HTML_COLLECTION, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_ALIGN,
            g_param_spec_string ("align", /* name */
                "html_table_element_align", /* short description */
                "read-write  gchar * HTMLTableElement.align", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_BG_COLOR,
            g_param_spec_string ("bg_color", /* name */
                "html_table_element_bg_color", /* short description */
                "read-write  gchar * HTMLTableElement.bg_color", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_BORDER,
            g_param_spec_string ("border", /* name */
                "html_table_element_border", /* short description */
                "read-write  gchar * HTMLTableElement.border", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_CELL_PADDING,
            g_param_spec_string ("cell_padding", /* name */
                "html_table_element_cell_padding", /* short description */
                "read-write  gchar * HTMLTableElement.cell_padding", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_CELL_SPACING,
            g_param_spec_string ("cell_spacing", /* name */
                "html_table_element_cell_spacing", /* short description */
                "read-write  gchar * HTMLTableElement.cell_spacing", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_FRAME,
            g_param_spec_string ("frame", /* name */
                "html_table_element_frame", /* short description */
                "read-write  gchar * HTMLTableElement.frame", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_RULES,
            g_param_spec_string ("rules", /* name */
                "html_table_element_rules", /* short description */
                "read-write  gchar * HTMLTableElement.rules", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SUMMARY,
            g_param_spec_string ("summary", /* name */
                "html_table_element_summary", /* short description */
                "read-write  gchar * HTMLTableElement.summary", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_WIDTH,
            g_param_spec_string ("width", /* name */
                "html_table_element_width", /* short description */
                "read-write  gchar * HTMLTableElement.width", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomHTMLTableElementPrivate));*/
}

static void gdom_html_table_element_init(GdomHTMLTableElement* request)
{
    /*GdomHTMLTableElementPrivate* priv = GDOM_HTML_TABLE_ELEMENT_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

